\documentclass{article}
\usepackage{cp0809t}
\usepackage{palatino}
\usepackage{a4}
\usepackage[draft]{fixme}

%include polycode.fmt 
%format fst = "\p1"
%format p1  = "\p1"
%format snd = "\p2"
%format p2  = "\p2"
%format Left = "i_1"
%format Right = "i_2"
%format i1 = "i_1"
%format i2 = "i_2"
%format >< = "\times"
%format |-> = "\mapsto"
%format . = "\comp"
%format -|- = "+"
%format Either a b = a "+" b
%format (either a b) = "\alt{" a "}{" b "}"
%format (either (a) (b)) = "\alt{" a "}{" b "}"
%format (uncurry f) = "\uncurry{" f "}"
%format (uncurry f) = "\uncurry{" f "}"
%format (const f) = "\underline{" f "}"
%format cata = "\hcata"
% -- desactivados:
%%format (uncurry (f)) = "\uncurry{" f "}"

\title{
		    Cálculo de Programas
\\
		Trabalho Teórico-Prático (Parte I)
\\
		LCC --- Ano Lectivo de 2008/09
}

\author{
		J.N. Oliveira \& H.D. Macedo
\\
		\dium
\\
		Universidade do Minho
}


\date\mydate

\makeindex

\begin{document}

\maketitle

\tableofcontents

\newpage

\section{Preâmbulo}

A disciplina de Cálculo de Programas tem como objectivo principal ensinar a
progra\-mação de computadores como uma disciplina científica. Para isso parte-se de uma
álgebra da programação (conjunto de leis universais e seus corolários) e aplica-se
essa álgebra a problemas de programação por resolução de equações cujas
incógnitas são os próprios programas que se querem obter.

Por razões de ordem pedagógica, restringe-se a aplicação desta ciência ao desenvolvimento
de programas funcionais e usa-se a linguagem \Haskell\ como veículo de programação~\footnote{%
A álgebra da programação estende-se à lógica em geral, encarada como liguagem para especificar problemas
e construir soluções. Esta continuação desta disciplina aborda-se na unidade curricular
\emph{Métodos Formais em Engenharia de Software} do Mestado em Informática desta Universidade,
cf.
\begin{quote}
\texttt{http://www.di.uminho.pt/ensino/formacao-avancada/metodos-formais-em-engenharia-de-software}.
\end{quote}
}.

O presente trabalho tem por objectivo concretizar na prática os
objectivos da disciplina acima enunciados: (a) por um lado, colocar os alunos perante
problemas de programação que deverão ser resolvidos em \haskell; (b) por outro lado,
convidar os alunos a documentarem esses mesmos programas com os cálculos que
precederam a sua codificação.
Há ainda um terceiro objectivo: o de ensinar a produzir textos técnico-científicos de qualidade.

Para cumprir de forma integrada e simples os objectivos acima vamos recorrer a uma técnica de
programa\-ção dita \litp{literária} \cite{Kn92}, cujo princípio base é o seguinte:
\emph{a documentação de um programa deve coincidir com ele próprio}. Por outras palavras,
o código fonte e a sua documentação deverão constar do mesmo documento (ficheiro).
O texto que está a ler é um exemplo de \litp{programação literária}, como adiante se referirá.

\section{Como realizar o trabalho}
Este trabalho teórico-prático deve ser realizado por grupos com um máximo
de três alunos. Os detalhes da avaliação (datas para submissão do relatório e sua defesa oral)
são os que constam da página da disciplina na \emph{internet}, onde também se encontra o
ficheiro original do presente texto.
Recomenda-se uma abordagem equilibrada e participativa dos membros do
grupo de trabalho por forma a poderem responder às questões que serão colocadas
na defesa oral do relatório.

Em que consiste, então, o \emph{relatório} a que se refere o parágrafo anterior?
Deverá ser um texto de \litp{\emph{programação literária}} \cite{Kn92} que responda aos
requisitos e questões de programação que constam deste enunciado, mais adiante.

\section{Como documentar código \Haskell}
Na programação científica, como é aquela que se ensina nesta disciplina,
é tão importante produzir um bom programa, ou pacote de programas
(vulg.\ \emph{aplicação de `software'}) como produzir evidência sobre a forma como foi
concebido (decisões tomadas, etc).
Este aspecto faz com que a \emph{documentação} sobre um programa seja
tanto ou mais valiosa que o programa propriamente dito.
(Essa documentação é, afinal, o mais precioso e duradouro bem de que uma empresa
dispõe sempre que um autor de um produto de `software' seu a abandona.)

Um erro clássico dos hábitos tradicionais de programação é o de se escrever
a documentação só depois do respectivo programa \emph{estar pronto}.
Por via de regra, a documentação escrita nessa fase tardia é omissa e de má qualidade.
Isto porque muitos detalhes do desenvolvimento já estão esquecidos nessa altura, sendo
muitas vezes tudo escrito \emph{a correr}.

Todos são unânimes em recomendar que a documentação seja escrita
\emph{ao mesmo tempo que} o código, e para isso nada melhor do que
usar o \emph{mesmo ficheiro} de texto para registar código e documentação.
Como já se disse acima,
chama-se a esta estratégia \litp{\emph{programação literária}}, sendo o \Haskell\ uma
das linguagens em que tal estratégia é prática comum.

É o que se passa com o texto (PDF) que neste momento está a ler: foi gerado a
partir de um ficheiro ( \file{cp0809t.lhs} ) usando o processador de texto
\Latex\ \cite{Kn86}; mas, se submeter o mesmo ficheiro ao seu interpretador
de \Haskell\ preferido, verá que o código nele incluído será seleccionado,
carregado e executado.

Exemplo: vamos declarar de seguida os isomorfismos $in$ e $out$ associados
à definição de listas em \Haskell\ tal como foram definidos nas aulas 
desta disciplina.
Primeiro vamos dar um nome ao módulo que estamos a definir
\begin{code}
module Cp0809t where
import Cp
import Nat
import System.Time
import Graphics.SOE
\end{code}
aproveitando para indicar que ele depende de funções disponíveis nos módulos |Cp|,
|Nat|, etc.
Basta, então escrevermos:
\begin{code}

inl = either (const []) (uncurry (:))

out []    = Left ()
out (a:x) = Right(a,x)
\end{code}

Se se inspeccionar o ficheiro \file{cp0809t.lhs} vê-se
que todo o texto indentado acima está entre dois delimitadores,
\verb!\begin{code}! e \verb!\end{code}!.
É nisso que consiste o protocolo que o \Latex\ e o \Haskell\
estabelecem entre si: quando o \Latex\ encontra blocos 
\verb!\begin{code} ... \end{code}!, limita-se a mostrá-los em formato \emph{pretty-printed}.
Por seu lado, o \Haskell\ está preparado para ignorar tudo o que é texto
\Latex\ \emph{e só prestar atenção} ao que está dentro desses blocos.

Por exemplo, se se submeter \file{cp0809t.lhs} ao \GHCi~\footnote{%
Note que tem de ter acessível os ficheiros \file{Cp.hs}, etc que são importados.
Note ainda que deve invocar o \GHCi\ com as opções de comando
\texttt{-XMultiParamTypeClasses}.
}, ter-se-á evidência de que tudo correu bem,
\begin{verbatim}
GHCi, version 6.10.1: http://www.haskell.org/ghc/  
Loading package ghc-prim ... linking ... done.
Loading package integer ... linking ... done.
Loading package base ... linking ... done.
[1 of 3] Compiling Cp           ( Cp.hs, interpreted )
[2 of 3] Compiling Nat          ( Nat.hs, interpreted )
[3 of 3] Compiling Cp0809t      ( cp0809t.lhs, interpreted )
Ok, modules loaded: Cp0809t, Nat, Cp.

\end{verbatim}
podendo-se de imediato questionar o interpretador.

Por exemplo, pode pedir-se para este calcular |out ["a","b","c"]|, devendo-se obter
\begin{verbatim}
*Cp0809t> out ["a","b","c"]
Right ("a",["b","c"])
\end{verbatim}
etc.

Continuando, vamos agora declarar o combinador $\cata\_$ que nos permite
construir \emph{catamorfismos} de listas:
\begin{code}
cata g   = g . rec (cata g) . out   
\end{code}
onde
\begin{code}
rec f    = id -|- id >< f
\end{code}
Voltando de novo ao \GHCi, podemos inquirir qual o tipo de |cata|,
\begin{verbatim}
*Cp0809t> :t cata
cata :: forall c a. (Either () (a, c) -> c) -> [a] -> c
\end{verbatim}
qual o resultado da aplicação do catamorfismo
\begin{eqnarray}
	\cata{\alt{\kons 0}{(1+) \comp \p2}}
	\label{eq:070421a}
\end{eqnarray}
à lista acima,
\begin{verbatim}
*Cp0809t> cata (either (const 0) ((1+) . p2)) ["a","b","c"]
3
\end{verbatim}
etc.

\section{Como documentar cálculos de programas}
Nas aulas desta disciplina, sempre que se pretende derivar um programa
em \Haskell, ou raciocinar sobre ele, não se usa a notação \Haskell\
directamente, mas sim uma notação mais compacta, de feição matemática,
que permite abreviar texto a agilizar os cálculos.
A expressão (\ref{eq:070421a}) acima, por exemplo, é a que é usada
no cálculo de catamorfismos e não a expressão \Haskell\ que acima se
passou ao \GHCi. Se quisermos, podemos ver (\ref{eq:070421a}) como
o ``pretty-printing'' do referido texto em sintaxe \Haskell.

Assim, não vamos usar \litp{programação literária} para documentar os cálculos,
mas sim o próprio \Latex, que tem uma natural afinidade com a matemática.
De facto, a produção de texto em \Latex\ é feita declarando funções (designadas
''macros'') cujo resultado é o texto a imprimir. Veja-se o exemplo
do operador unário |cata|, que em \Latex\ é a macro unária do mesmo
nome declarada no ficheiro auxiliar \file{cp0809t.sty}.

A inspecção deste ficheiro revelará ao leitor a vantagem de se reproduzir
na linguagem do processador de texto a estrutura da linguagem matemática
que estamos a usar.
%\footnote{Em particular, a secção \emph{Importing/modifying isolatin1} mostra como usar caracteres especiais para esse efeito.}.
Isso revela-se em particular no que diz respeito
à apresentação dos cálculos, tal como o que se segue, extraído
``verbatim'' dos apontamentos teóricos da disciplina~\cite{Ol05}:
\begin{calculation}
%
     swap \comp swap
%
\just={by definition $swap \deff \split{\p2}{\p1}$}
%
        \split{\p2}{\p1} \comp swap
%
\just={by $\times$-fusion (\ref{eq:702e})}
%
        \split{\p2 \comp swap}{\p1 \comp swap}
%
\just={definition of $swap$ twice}
%
        \split{\p2 \comp \split{\p2}{\p1}}
	      {\p1 \comp \split{\p2}{\p1}}
%
\just={by $\times$-cancellation (\ref{eq:700l})}
%
        \split{\p1}{\p2}
%
\just={by $\times$-reflexion (\ref{eq:703a})}
%
        id
%
\end{calculation}

\noindent
A inspecção de file{cp0809t.lhs} revelará uma estrutura
\begin{verbatim}
\begin{calculation}
%
     swap \comp swap
%
\just={ .................. }
%
	......
%
\just={ .................. }
%
        id
%
\end{calculation}
\end{verbatim}
cujo `layout' se aproxima bastante do que é produzido e onde desempenha papel
muito relevante a \emph{macro} binária \texcmd{just} (de \emph{``justificação''})
cujo primeiro argumento é o símbolo a justificar (igualdade de funções, neste caso)
e o segundo é o texto justificativo.

\section{O que se pretende}
O anexo \ref{sec:exercs} deste texto contém um enunciado de questões várias
sobre a matéria da disciplina, umas de natureza teórica
e outras de natureza prática, devendo resultar das primeiras cálculos de
programas em estilo \emph{point-free} e das últimas programas em \Haskell\
que deverão poder ser testados num interpretador de \Haskell.
Os alunos devem responder a cada pergunta escrevendo o texto \lhaskell{lhs}
\litp{(``literate haskell'')}
que entenderem e como entenderem, ora escrevendo os seus cálculos como acima se
explicou, ora embebendo código \Haskell\ no seu texto.
Para imprimir o relatório basta usar a sequência de comandos
\begin{verbatim}
lhs2TeX -o cp0809t.tex cp0809t.lhs
pdflatex cp0809t
\end{verbatim}
(onde se usa \file{cp0809t.lhs} como exemplo) em que o papel essencial é
desempenhado pelo pre-processador \LhsToTex~\footnote{%
Ver \verb"www.informatik.uni-bonn.de/~loeh/lhs2tex".
}.

Sugere-se o aproveitamento da fonte \Latex\ deste documento (\file{cp0809t.lhs})
como ponto de partida. O ficheiro fonte do relatório deverá ser enviado
à equipa docente seguindo as instruções que serão publicadas na
página da disciplina.

Os alunos deverão ainda aproveitar este texto para mostrar que sabem manipular
biliografias (em \Bibtex) e índices/glossários (usando \Makeindex).
Para se conhecer a vastíssima documentação disponível sobre \Latex,
\Bibtex, etc, etc, basta escrever \texttt{"latex"}, \texttt{"bibtex"},
\texttt{"makeindex"} etc num motor de busca e começar a navegar.
Ou então visitar o \TUG\ (\emph{\TeX\ Users Group}) em
\texttt{www.tug.org/}.

\section{Desenho de diagramas}
Há muitos \emph{pacotes \Latex} para desenhar diagramas e outros objectos
gráficos \cite{GRM97}. Sugere-se, entre estes,
o \xypic{XY-pic}~\footnote{ Ver \texttt{http://www.tug.org/applications/Xy-pic/}.},
pela sua simplicidade e pelo facto de constar das principais distribuições
de \Latex. Como exemplo, dá-se o diagrama,
\begin{eqnarray*}
\xymatrix{
	B
		\ar[d]_-{f}
&
	A
		\ar[l]_-{g}
		\ar[dl]^-{f \comp g}
\\
	C
}
\end{eqnarray*}
extraído de  \cite{Ol05}.
Bastará inspeccionar o ficheiro \file{cp0809t.lhs} para
ver quão simples é desenhar diagramas como este.

\appendix

\newpage

\section{Exercícios a resolver}\label{sec:exercs}
Como orientação geral para a resolução das questões que se seguem,
considerem que as respostas (sejam elas cálculos, provas, código \Haskell,
etc) serão tanto mais valorizadas quanto \textbf{mais simples} forem.
No caso de código \Haskell, procurem usar combinadores que reduzam o número
de linhas de código.

O trabalho consta da realiza\c c\~ao de 10 exerc\'icios, disponibilizados em duas fases
para permitir maior flexibilidade na sua resolu\c c\~ao.

Seguem-se 10 propostas de exerc\'icios a resolver das quais se devem resolver
8, sendo 5 obrigat\'orias (marcadas com o sinal ``\obrigatorio''). As restantes
s\~ao \`a escolha de cada grupo.
As valorizaÁıes de questıes ou al\'ineas obrigat\'orias, quando existam, s„o naturalmente opcionais.

\traco

\begin{exercise} 
Demonstre a lei de fusão do coproduto, cf.\ (\ref{eq:+-fusion}) no Anexo
\ref{sec:FunCalcfragment}, pág.~ \pageref{eq:+-fusion}, desenhando o diagrama
respectivo.
\end{exercise}

\begin{exercise}\obrigatorio
Relembre as aulas em que estou a aplica\c c\~ao do algoritmo de Hindley-Milner
\`a infer\^encia de tipos polim\' orficos em Haskell. Aplique os passos desse algoritmo
na dedu\c c\~ao do tipo
\begin{quote}
|for :: (Integral n) => (a -> a) -> a -> n -> a|
\end{quote}
que o \GHCi\ devolve para o combinador |for| que a biblioteca \file{Nat.hs}
oferece como ``fold'' associado aos n\'umeros naturais.
\end{exercise}

\begin{exercise}\obrigatorio
Sempre que um gene de um catmorfismo \'e, como acontece na lei de recursividade m\'ultipla (\ref{eq:fokkinga}),
um ``split'' de duas fun\c c\~oes, designa-se esse caso particular de catamorfismo um \textbf{mutumorfismo}
e usa-se uma nota\c c\~ao especial para o identificar, tal como se segue:
\begin{eqnarray}
	 \mutu h k  & \deff & \cata{\split h k}
	\label{eq:mutudef}
\end{eqnarray}

Repare que a lei de de recursividade m\'ultipla (\ref{eq:fokkinga}) pode
ser vista como a propriedade universal do combinador $\mutu{}{}$, lei essa
que, ali\'as, se pode ainda desenvolver mais usando propriedades de produtos
(sec\c c\~ao \ref{sec:Produto}).

Pretende-se neste exerc\'icio avaliar a sua agilidade na dedu\c c\~ao de novas leis do c\'alculo de programas
a partir das j\'a conhecidas. Nesse sentido, derive as leis de cancelamento, fus\~ao e reflex\~ao para
mutumorfismos.
\end{exercise}

\begin{exercise} 
Demonstre a seguinte igualdade onde se apresentam duas formas de calcular a mesma função:
\begin{equation}
for \, (a+) \, 0 = \p2 \comp for \, (\split{\p1}{\trans +}) \, (a,0)
\end{equation}
Que função é essa?  
\end{exercise}

\begin{exercise} 
Dado que todos os catamorfismos sobre naturais são da forma $\cata{\alt{\kons k} g}$ 
mostre que $\cata{\alt{\kons k}g} \, n = g^n k$, onde
$g^n$ é $n$-ésima iteração de $g$, i.e.,
$g^0 = id$ e $g^{n+1}=g \comp g^n$. Note que $g$ é o corpo de um ``ciclo-for''
que se repete $n$-vezes, começando com o valor inical $k$.
\end{exercise}

\begin{exercise}\obrigatorio
A função coseno $\cos x : \R \rightarrow [-1,1]$ pode ser definida de várias maneiras, entre elas aquela que toma
a forma de uma série de Taylor:

\begin{eqnarray}
	  \cos x 
& = &
      \sum^{\infty}_{n=0} \frac{(-1)^n}{(2n)!} x^{2n}\quad \forall  x\! 
\end{eqnarray}
A seguinte função, em \Haskell,
\begin{code}
coseno :: Double -> Integer -> Double
coseno x 0 = 1
coseno x (n+1) = sig (n+1) * (pow2n x (n+1)) /  (fac2n (n+1)) 
					+ coseno x n
\end{code}
calcula uma aproximação a $cos x$, onde o parâmetro adicional indica o número
de parcelas da série de Taylor que estão a ser tomadas em consideração.
Por exemplo, enquanto que |coseno 1 1 = 0.5|, já |coseno 1 10| dá o resultado
|0.5403023058681397|.

Recorde das aulas práticas que podemos ``linearizar'' a função $x^{2n}$ definindo 
uma nova função $pow2n\,n = x^{2n}$ e desenvolvendo em
\begin{code}
pow2n x 0 = 1
pow2n x (n+1) = x * x * (pow2n x n)

\end{code}
Que por sua vez é um catamorfismo de naturais, e a função sig também se pode exprimir
através de um catamorfismo no mesmo tipo de dados:
\begin{code}
sig = cataNat (either (const 1) ((-1)*))

\end{code}
sendo aqui expressa na sua forma minimal. Já a função que calcula o factorial de $2 n$ pode 
ser também definida por recursão no tipo de dados Nat usando uma outra versão do catamorfismo.
\begin{code}
fac2n = p2 . fcata 
	    where
	     fcata 0 = (0,1)
	     fcata (n+1) = let (a,b) = fcata n 
	                   in (a+1,((2*a)+2) * ((2*a)+1) * b)

	
\end{code}
Contudo, a função |coseno| não é muito eficiente.  Se recorrer às funções
disponíveis na package |System.Time| para contagem de tempo de CPU verifica
que, enquanto o cálculo de |coseno 1 100| é quase instantâneo, o de |coseno 1 1000|
leva cerca de 4 segundos, o de |coseno 1 2000| 22 segundos, etc

	Apresente em detalhe todos os cálculos que deverá fazer para, recorrendo a
	leis como a de recursividade múltipla e/ou o seu corolário \emph{banana-split},
	obter uma versão mais eficiente de |coseno x n|.
%\item
%	Apresente resultados de testes de execução das duas funções (variando o
%	valor de |n|) por forma a documentar o ganho em eficiência.
%	Para esse efeito, transforme-as em funções sobre o mónade de \IO\ e recorra,
%	nesse contexto, a funções como |getClockTime|, |diffClockTimes| etc.\ da
%	package |System.Time| que o ajudem nessa tarefa. \textbf{(Opcional)}
\end{exercise}

\begin{exercise}\obrigatorio
Para resolver este exerc\'{\i}cio dever\'a ler primeiro o ap\^endice
\ref{sec:090612a} e, em particular, o que se refere ao \textbf{triângulo de Sierpinski}:
\begin{enumerate}
\item Desenvolva a biblioteca \emph{pointfree} \texttt{TLTree.hs} de forma análoga
      a outras bibliotecas que conhece (\eg\ \file{BTree.hs}, \ldots).
\item Exprima as funções \verb+geraSierp+ e \verb+apresentaSierp+ como
      anamorfismo e catamorfismo, respectivamente, do tipo \verb+TLTree+.
\item Recorrendo às funções relevantes da biblioteca gráfica, escreva
  uma função 
\begin{verbatim}
desenha :: [Tri] -> IO() 
\end{verbatim}
que desenha uma lista de triângulos.
\item Teste o seu programa:
\begin{verbatim}
desenha (sierpinski (50,300) 256 5)
\end{verbatim}
\end{enumerate}
\end{exercise}

\begin{exercise}
	Atente no módulo Google apresentado no apéndice \ref{sec:mapreduce}.
	\begin{enumerate}
	\item Converta a função \textbf{groupByKey} para pontwise e apresente a sua definição em Haskell.
	\item Recorrendo às leis de (\ref{eq:cata-absorption}) consegue comprimir algumas das definções apresentadas no modulo Google?
    \end{enumerate}
\end{exercise}

\begin{exercise}
	Pretende-se implementar um motor de busca para um sistema de ficheiros usando o modelo MapReduce. Suponha para uma 
	dada pesquisa X (uma ou mais palavras) o resultado é a lista dos ficheiros onde essa(s) palavra(s) X ocorre(m) ordenada 
	por número de ocorr\^encias. 
	
	Para isso precisamos de construir uma função  
	\begin{quote}
	|makeIndex :: [(NomeFicheiro,Conteudo)] -> [(Palavra,[(NomeFicheiro,Int)])]|
	\end{quote}
	que, dados os nomes de ficheiro e o seu conteúdo (texto), pré-compute para cada palavra contida nesses ficheiros a lista dos 
	nomes de ficheiro onde aparece e o número de vezes.
  	
	Identifique os parametros m(ap) e r(educe) que calculam esta função.
\end{exercise}


\begin{exercise}\obrigatorio
Na sec\c{c}\~ao (5) da biblioteca \file{LTree.hs} declara-se o tipo |LTree a| como
inst\^ancia da classe |Monad|. Apresente c\'alculos que provem que as propriedades
(\ref{eq:mon-mul}) e (\ref{eq:mon-unit}) s\~ao de facto v\'alidas, para este m\'onade.
\\
\textbf{Valoriza\c{c}\~ao}: poder\'a optar por provar que, em geral, qualquer tipo
cuja base \'e $\b(f,g) = f + \ff g$ (onde $\f$ \'e um functor arbitr\'ario) forma
um m\'onade, para $\mu=\cata{\alt{id}{in \comp i_2}}$ e $\u=in \comp i_1$.
J\'a agora: que outros m\'onades estudou que caem nesta classe?
\end{exercise}

\newpage

\section{Gr\'aficos em Haskell}
\label{sec:090612a}
\subsection{Fractais}

Um \emph{fractal} é uma estrutura matemática que se repete
infinitamente, e a que corresponde uma representação visual
``infinitamente detalhada''. Uma imagem possível é a seguinte: se
usarmos dispositivos de ampliação cada vez mais potentes (lupas,
microscópios), conseguiremos sempre ver cada vez mais detalhes na
figura, que pareciam não existir quando a observávamos com o
dispositivo anterior. Os fractais têm aplicações importantes que não
serão aqui exploradas, nomeadamente na simulação de fenómenos
naturais (por exemplo climatéricos).

Neste trabalho estudar-se-á a geração de fractais por hilomorfismos de
determinados tipos indutivos.

\subsection{A Biblioteca Gráfica \texttt{Graphics.SOE}}

Para o desenho dos fractais sugere-se a utilização da biblioteca
\texttt{Graphics.SOE}~\cite{hudak00haskell}\footnote{distribuída como
  parte da \emph{Hugs Graphics Library} em
  \texttt{http://haskell.org/graphics/}}. Para isso basta adicionar
\begin{verbatim}
import Graphics.SOE
\end{verbatim} como já foi feito.
O desenho de objectos gráficos não é mais do que um caso especial de
processamento de \emph{input / output} monádico. Apresentam-se aqui
algumas funções básicas suficientes para o presente trabalho.

Antes de mais, vejamos como criar uma janela gráfica utilizando a
seguinte função:
\begin{verbatim}
type Title = String
type Size = (Int,Int)
openWindow :: Title -> Size -> IO Window
\end{verbatim}

O seguinte programa abre uma janela com nome ``Sierpinski'', de
dimensão 300 $\times$ 300; escreve nela uma mensagem; espera que uma
tecla seja premida; finalmente fecha a janela.
\begin{code}
main =
   runGraphics (
   do w <- openWindow "Sierpinski" (300,300)
      drawInWindow w (text (100,200) "Aqui está uma janela")
      k <- getKey w
      closeWindow w
   )
\end{code}
A função \verb+drawInWindow :: Window -> Graphic -> IO()+ desenha um
objecto de tipo \verb+Graphic+ numa janela.  É naturalmente possível
desenhar outro tipo de objectos além de texto; o seguinte será útil
para o desenho dos fractais:
\begin{verbatim}
type Point = (Int,Int)
polygon :: [Point] -> Graphic
\end{verbatim}
Objectos gerados por esta função correspondem a polígonos descritos
por uma lista de pontos.

Para colorir os fractais sugere-se a utilização da seguinte função:
\begin{verbatim}
withColor :: Color -> Graphic -> Graphic
\end{verbatim}

Como exemplo de utilização, apresenta-se uma função que desenha numa
janela um triângulo rectângulo isósceles de cor azul, descrito pelas
coordenadas de um dos seus vértices e pelo comprimento dos seus
catetos:
\begin{code}
fillTri :: Window -> Int -> Int -> Int -> IO()
fillTri w x y size = drawInWindow w (withColor Blue
                      (polygon [(x,y),(x+size,y),(x,y-size),(x,y)]))
\end{code}

Uma nota final: o ponto $(0,0)$ numa janela gráfica corresponde ao
canto \emph{superior esquerdo} da mesma. 

\subsection{O Triângulo de Sierpinski}



\begin{figure}[htb]
\begin{center}
\includegraphics{sierpinski2.eps}
\end{center}
  \caption{Construção de um triângulo de Sierpinski}
  \label{fig:sierp1}
\end{figure}


O triângulo de Sierpinski é uma figura com carácter fractal que se
obtém da seguinte forma: considere-se um triângulo rectângulo e
isósceles $A$ cujos catetos têm comprimento $s$. A estrutura fractal é
criada desenhando-se três triângulos no interior de $A$, todos eles
rectângulos e isósceles e com catetos de comprimento $s/2$. Este passo
é depois repetido para cada um dos triângulos desenhados, e assim
sucessivamente. Os três primeiros passos são apresentados na
Fig.~\ref{fig:sierp1}.

Note-se que um triângulo de Sierpinski é gerado repetindo-se
infinitamente o processo acima descrito; no entanto para efeitos de
visualização num monitor (de resolução forçosamente finita) será
necessário escolher uma representação adequada do triângulo, parando o
processo recursivo a um determinado nível. A figura a desenhar é
constituída por um conjunto finito de triângulos todos da mesma
dimensão (por exemplo, no quarto triângulo da figura, desenhar-se-iam
27 triângulos). Cada triângulo é geometricamente descrito pelas
coordenadas do seu vértice inferior esquerdo e o comprimento dos seus
catetos:
\begin{code}
type Side = Int
type Tri = (Point, Side)
\end{code}

\subsection{Criação de um Triângulo de Sierpinski por um Hilomorfismo}

A estrutura recursiva de (uma representação finita de) um triângulo de
Sierpinski é captada por uma árvore ternária (cada nó é um triângulo
com os respectivos três sub-triângulos). Nas folhas dessa árvore
encontram-se os triângulos mais pequenos (todos da mesma dimensão),
que deverão ser desenhados. Apenas estes conterão informação de
carácter geométrico, tendo os nós da árvore um papel exclusivamente
estrutural.

\begin{code}
data TLTree = Leaf Point Side 
           | Node TLTree TLTree TLTree
\end{code}
A informação geométrica guardada em cada folha consiste nas
coordenadas do vértice inferior esquerdo e no lado dos catetos do
respectivo triângulo.

A função $\verb+sierpinski :: Point -> Side -> Int -> [Tri]+$ recebe
as coordenadas do vértice inferior esquerdo e a dimensão do triângulo
exterior, bem como o número de níveis pretendido (critério de paragem
do processo de construção do fractal), e constrói uma lista de
triângulos a desenhar.  Esta função é um hilomorfismo do tipo
\verb+TLTree+, composição das seguintes duas funções:
\begin{code}
sierpinski :: Point -> Side -> Int -> [Tri]
sierpinski p s = apresentaSierp . (geraSierp p s)

geraSierp :: Point -> Side -> Int -> TLTree
geraSierp (x,y) s 0 = Leaf (x,y) s
geraSierp (x,y) s n = Node (geraSierp (x,y) (s `div` 2) (n-1))
                           (geraSierp (x+s`div`2,y) (s `div` 2) (n-1))
                           (geraSierp (x,y-s`div`2) (s `div` 2) (n-1))

apresentaSierp :: TLTree -> [(Point, Side)]
apresentaSierp (Leaf (x,y) s) = [((x,y),s)]
apresentaSierp (Node a b c) =
             (apresentaSierp a)++(apresentaSierp b)++(apresentaSierp c)  
\end{code}

\newpage

\section{O modelo MapReduce}\label{sec:mapreduce}

A seguir apresenta-se um módulo contendo uma implementação do modelo de programação MapReduce tornado popular 
pela Google \cite{La07}. Resumidamente este modelo baseia-se em aplicar uma função a vários elementos de uma lista de pares
chave/valor, agrupar os resultados por chave e finalmente calcular resultados finais para cada para cada 
elemento resultante da fase de agrupamento.  

Note-se que,
mesmo estando entre blocos \verb!\begin{code}% ... \end{code}!,
esta parte não vai ser considerada pelo interpretador/compilador de Haskell
(via \LhsToTex) por causa do símbolo de percentagem % \textbf{\%}
extra. Esta variante permite esconder esta parte do interpretador/compilador
de Haskell e ao mesmo tempo apresent\'a-la em \Latex. Recorremos a este truque
para evitar um ficheiro com duas definições de módulos, algo que não
\'e permitido.

\begin{code}% 
	
module Google where

-- Adatped from Google's MapReduce programming model revisited
-- by Ralf Laemmel, DOI: http://dx.doi.org/10.1016/j.scico.2007.07.001 


import Cp

mapReduce :: (Eq b, Eq a1) =>
             (a2 -> b2 -> [(a1, b)])
             -> (a1 -> [b] -> Maybe b1)
             -> [(a2, b2)]
             -> [(a1, b1)]
mapReduce m r =
     (reducePerKey r)         -- 3. Apply *r* to each group
   . groupByKey               -- 2. Group intermediate data per key
   . (mapPerKey m)            -- 1. Apply *m* to each key/value pair

mapPerKey f =
      concat                  -- 2. Concatenate per-key lists
    . map (uncurry f)         -- 1. Map *f* over list of pairs

groupByKey :: (Eq b, Eq a) => [(a, b)] -> [(a, [b])]
groupByKey = foldr ins []
             where ins (a,v) m = consWith (++) a [v] m

reducePerKey r =
      (mapWithKey unJust)     -- 3. Transform type to remove Maybe
    . filterWithKey isJust    -- 2. Remove entries with value Nothing
    . (mapWithKey r)          -- 1. Apply *r* per key
   where 
         unJust k x = gJust x -- Transforms optional into non-optional type
         isJust k (Just v)   = True    -- Keep entries of this form
         isJust k Nothing    = False   -- Remove entries of this form
\end{code}

\begin{code}%
-- Example:

main = print 
     $ wordOccurrenceCount 
     $ insert "doc2" " appreciate the unfold " 
     $ insert "doc1" " fold the fold " 
     $ []

-- where 

wordOccurrenceCount = mapReduce m r 
     where m = const ( map ( flip (,) 1) . words) -- each word counts as 1 
           r = const (Just . sum)                 -- compute sum of all counts
	   sum = foldl (+) 0
\end{code}


\begin{code}%
-- Auxiliary functions

gJust (Just v)   = v       -- Get contents of Maybe

consWith :: (Eq b, Eq a) => (a -> a -> a) -> b -> a -> [(b, a)] -> [(b, a)]
consWith f k v m
         | lookup k m == Nothing = (k,v):m
         | otherwise                = (k,f v z):(filter ((/=k).fst) m)
				      where z = gJust (lookup k m)

insert :: (Eq a, Eq b) => b -> a -> [(b, a)] -> [(b, a)]
insert = consWith const

filterWithKey :: (a -> b -> Bool) -> [(a, b)] -> [(a, b)]
filterWithKey p = filter (uncurry p)

mapWithKey :: (a -> b -> c) -> [(a, b)] -> [(a, c)]
mapWithKey f = map (split fst (uncurry f))
\end{code}


\newpage
\section{Algumas leis do cálculo funcional}\label{sec:FunCalcfragment}

\subsection{Produto}\label{sec:Produto}
\begin{eqnarray}
\law{splitUniv}{
        Universal-$\times$
}{
	% diagrama
}{
k = \split f g & \equiv &
        \left\{
                \begin{array}{rcl}
                        \p1 \comp k = f
                \\
                        \p2 \comp k = g
                \end{array}
        \right.
}
\\
%
\law{700l}{
        Cancelamento-$\times$
}{
        diagrama
}{
        \p1 \comp \split f g  = f
        & \hbox{,} &
        \p2 \comp \split f g = g 
}
\\
\law{703a}{
	Reflexão-$\times$
}{
	diagrama
}{
	\split{\p1}{\p2} &=& id_{A\times B}
}
\\
\law{702e}{
        Fusão-$\times$
}{
        \xymatrix{
        A & A \times B \ar[l]_{\p1} \ar[r]^{\p2} & B
        \\ & C \ar[ul]^{g} \ar[u]^{\split g h} \ar[ur]_{h}
        \\ & D \ar[uul]^{g \comp f} \ar[u]^{f} \ar[uur]_{h \comp f}
}}{
        \split{g}{h} \comp f = \split{g \comp f}{h \comp f}
}
\\
\law{970511c}{
        Absorção-$\times$
}{
        \xymatrix{
           A & A \times B \ar[l]_{\p1} \ar[r]^{\p2} & B
        \\ D \ar[u]^{i} &
           D \times E \ar[l]_{\p1} \ar[r]^{\p2} \ar[u]^{i\times j} &
           E \ar[u]^{j}
        \\ & C \ar[ul]^{g} \ar[u]^{\split{g}{h}} \ar[ur]_{h}
}}{
	(i\times j) \comp \split{g}{h} = \split{i \comp g}{j \comp h}
}
\\
\law{extra1}{
        Def-$\times$
}{
diagrama
}{
        f \times g  = \split{f \comp \p1}{g \comp \p2}
}
\\
\law{extra5}{
        Natural-$\p1$
}{
}{
       \p1 \comp (f \times g) &=& f \comp \p1
}
\\
\law{extra6}{
        Natural-$\p2$
}{
}{
       \p2 \comp (f \times g) &=& g \comp \p2
}
\\
\law{705h}{
        Functor-$\times$
}{
}{
       (g \comp h) \times (i \comp j) &=&
       (g \times i) \comp (h \times j)
}
\\
\law{705j}{
        Functor-id-$\times$
}{
}{
        id_A \times id_B &=& id_{A\times B}
}
\\
\law{extra2}{
        Eq-$\times$
}{
diagrama
}{
        \split{f}{g}  = \split{h}{k} \; \equiv\; f=h \land g=k
}
\end{eqnarray}

\subsection{Coproduto}
\begin{eqnarray}
\law{eitherUniv}{
        Universal-$+$
}{
	% diagrama
}{
k = \alt f g & \equiv &
        \left\{
                \begin{array}{rcl}
                        k \comp i_1 = f
                \\
                        k \comp i_2 = g
                \end{array}
        \right.
}
\\
\law{701c}{
        Cancelamento-$+$
}{
        \xymatrix{
        A \ar[r]^{i_1} \ar[dr]_{g} &
        A+B \ar[d]^{\alt g h} &
        B \ar[l]_{i_2} \ar[dl]^{h}
        \\ & C
}
}{
        \alt g h \comp i_1 = g & , & \alt g h \comp i_2 = h
}
\\
\law{703b}{
        Reflexão-$+$ 
}{
        diagrama
}{
                \alt{i_1}{i_2} &=& id_{A + B}
}
\\
\law{+-fusion}{
        Fusão-$+$
}{
        \xymatrix{
        A \ar[r]^{i_1} \ar[dr]_{g} \ar[ddr]_{f\comp g} & A+B \ar[d]^{\alt g h} &
        B \ar[l]_{i_2} \ar[dl]^-{h} \ar[ddl]^{f\comp h}
        \\ & C \ar[d]^{f}
        \\ & D
}
}{
        f \comp \alt g h = \alt{f \comp g}{f \comp h}
}
\\
\law{+-absorption}{
        Absorção-$+$
}{
        \xymatrix{
           A \ar[r]^{i_1} \ar[d]_{i} & A+B \ar[d]_{i+j} & B \ar[l]_{i_2} \ar[d]^{j} &
        \\ D \ar[r]^{i_1} \ar[dr]_{g} & D+E \ar[d]^{\alt g h} &
           E \ar[l]^{i_2} \ar[dl]^{h}
        \\ & C
}
}{
        \alt g h \comp (i + j)  = \alt{g \comp i}{h \comp j}
}
\\
\law{extra3}{
        Def-$+$
}{
diagrama
}{
        f + g  = \alt{i_1 \comp f}{ i_2 \comp g}
}
\\
\law{706b}{
        Functor-$+$
}{
}{
        (g \comp h) + (i \comp j) = (g + i) \comp (h + j)
}
\\
\law{706d}{
        Functor-id-$+$
}{
}{   
        id_A + id_B = id_{A + B}
}
\\
\law{extra4}{
        Eq-$+$
}{
diagrama
}{
        \alt{f}{g}  = \alt{h}{k} \; \equiv\; f=h\, \land\, g=k
}
\\
\law{eq:701d}{
        Lei da troca
}{
}{
        \alt{\split f g}{\split h k} &=&
        \split{\alt f h}{\alt g k}
}
\end{eqnarray}

\subsection{Indução}
\begin{eqnarray}
\law{cataUniv}{
        Universal-cata
}{
}{
        k = \cata\beta  & \equiv & k  \comp in = \beta   \comp (\ff k)
}
\\
\law{cata-cancel}{
        Cancelamento-cata
}{
}{
        \cata\alpha  \comp in = \alpha  \comp \ff{\cata\alpha}
}
\\
\law{cata-reflex}{
        Reflexão-cata
}{
        \xymatrix{
        \muF  \ar[d]_-{\cata{in}} &
        \ap\f{\muF } \ar[d]^{\ff{\cata{in}}} \ar[l]_-{in}
\\
        \muF  & \ap\f{\muF } \ar[l]^-{in}
}
}{
        \cata{in} = id_{\muF }
}
\\
\law{cata-fusion}{
        Fusão-cata
}{
        \xymatrix{
        \muF  \ar[d]_-{\cata\alpha} &
        \ap\f{\muF } \ar[d]^{\f\cata\alpha} \ar[l]_-{in}
\\
        A \ar[d]_-{f} & \ff A \ar[l]^-\alpha \ar[d]^{\ff f}
\\
        B & \ff B \ar[l]^-\beta
}
}{
        f  \comp \cata\alpha = \cata\beta
        & \implied &
        f  \comp \alpha = \beta  \comp (\ff f)
}
\\
\law{cata-absorption}{
        Absorção-cata
}{
}{
	\cata{\alpha}  \comp \tf f & = & \cata{\alpha  \comp \bb f {id}}
}
\\
\law{def-map}{
        Def-map
}{
}{
	\tf f & = & \cata{in_\f  \comp \bb f {id}}
}
\end{eqnarray}

\subsection{Recursividade m\'utua}
\begin{eqnarray}
\law{fokkinga}{
        Fokkinga
}{
}{
\begin{lcbr}
	f  \comp in = h  \comp \ff{\split f g}
\\
        g  \comp in = k  \comp \ff{\split f g}
\end{lcbr}
	& \equiv &
\split f g = \cata{\split h k}
}
\\
\law{bananaSplit}{
        ``Banana-split''
}{
}{
	\split{\cata i}{\cata j} =
	\cata{(i \times j)  \comp \split{\ff{\p1}}{\ff{\p2}}}
}
\end{eqnarray}

\begin{comment}
\subsection{Exponenciação}
\begin{eqnarray}
\law{transUniv}{
        Universal-exp
}{
}{   
	k = \trans f & \equiv & f = ap \comp (k \times id)
}
\\
\law{999f}{
        Cancelamento-exp
}{
        \xymatrix{
           B^A & B^A \times A \ar[r]^{ap} & B
        \\ C \ar[u]^{\trans f} &
           C \times A \ar[u]^{\trans f \times id} \ar[ru]_{f}
}
}{
        f = ap \comp (\trans f \times id)
}
\\
\law{exp-reflex}{
        Reflexão-exp
}{
        \xymatrix{
           B^A & B^A \times A \ar[r]^{ap} & B
        \\ B^A \ar[u]^{id_{B^A}} &
           B^A \times A \ar[u]^{id_{B^A} \times id_A} \ar[ru]_{ap}
}
}{
        \trans{ap} = id_{B^A}
}
\\
\law{exp-fusion}{
        Fusão-exp 
}{
	omitido
}{
        \trans{g\comp(f \times id)} = \trans{g} \comp f
}
\\
\law{999h}{
        Absorção-exp 
}{
	omitido
}{
	f^A \comp \trans g = \trans{f \comp g}
}{
}
\\
\law{080328a}{
        Def-exp 
}{
}{
	f^A = \trans{f \comp ap}
}{
}
\\
\law{expFun}{
        Functor-exp 
}{
}{
        (g \comp h)^A = g^A \comp h^A
}
\\
\law{expFunId}{
        Functor-id-exp
}{
}{
        id^A = id
}
\end{eqnarray}
\end{comment}

\subsection{Mónades}
\begin{eqnarray}
\law{mon-mul}{
        Multiplicação
}{
}{
	\mu \comp \mu = \mu \comp \ff \mu
}
\\
\law{mon-unit}{
        Unidade
}{
}{
	\mu \comp \u = \mu \comp \ff \u = id
}
\end{eqnarray}



%----------------- Bibliografia (exige bibtex) --------------------------------%
\newpage
\bibliographystyle{plain}
\bibliography{cp0809t}
%----------------- Índice remissivo (exige makeindex) -------------------------%
\printindex
\end{document}
